#include "I2C-LS.h"


uint8_t slaveAdr = I2C_LS_ADR_J1_GND;
/**********************************************************
Konstruktor
**********************************************************/
I2C_Lichtsensor::I2C_Lichtsensor()
{}

/**********************************************************
ffentliche Methoden
**********************************************************/

/*---------------------------------------------------------
Name:           begin

Beschreibung:   initialisiert den I2C-LS
 
Eingnge:       void

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::begin(void)
{
  Wire.begin();
  slaveAdr = I2C_LS_ADR_J1_GND;
  reset();
}

/*---------------------------------------------------------
Name:           begin

Beschreibung:   initialisiert den I2C-LS
 
Eingnge:       uint8_t adr
                 SlaveAdresse des I2C-LS

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::begin(uint8_t adr)
{
  Wire.begin();
  if( (adr == I2C_LS_ADR_J1_GND)
    || (adr == I2C_LS_ADR_J1_VCC)
    || (adr == I2C_LS_ADR_J1_SCL)
    || (adr == I2C_LS_ADR_J1_SDA) )
  {
      slaveAdr = adr;
  }
  reset();
}

/*---------------------------------------------------------
Name:           reset

Beschreibung:   reset des I2C-LS
 
Eingnge:       void

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::reset(void)
{
  Wire.beginTransmission(0);
  Wire.write(0x06);
  Wire.endTransmission();
}

/*---------------------------------------------------------
Name:           writeRegister

Beschreibung:   schreibt ein Register des I2C-LS
 
Eingnge:       uint8_t regAddress
                  Registeradresse an der die Daten geschrieben
                  werden sollen

                uint16_t data
                  zu schreibende Daten

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::writeRegister(uint8_t regAddress, uint16_t data)
{
  Wire.beginTransmission(slaveAdr);

  Wire.write(regAddress);

  /* Write */
  Wire.write((uint8_t)(data >> 8));
  Wire.write((uint8_t)(data & 0x00FF));

  Wire.endTransmission();
}

/*---------------------------------------------------------
Name:           writeRegister

Beschreibung:   schreibt ein Register des I2C-LS
 
Eingnge:       I2CLSRegister_t regAddress
                  Registeradresse an der die Daten geschrieben
                  werden sollen

                uint16_t data
                  zu schreibende Daten

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::writeRegister(I2C_LSRegister_t regAddress, uint16_t data)
{
  Wire.beginTransmission(slaveAdr);

  Wire.write((uint8_t)regAddress);

  /* Write */
  Wire.write((uint8_t)(data >> 8));
  Wire.write((uint8_t)(data & 0x00FF));

  Wire.endTransmission();
}

/*---------------------------------------------------------
Name:           readRegister

Beschreibung:   liest ein Register des I2C-LS
 
Eingnge:       uint8_t regAddress
                  Registeradresse von der die Daten gelesen
                  werden sollen

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readRegister(uint8_t regAddress)
{
  uint8_t lsb;
  uint8_t msb;
  
  Wire.beginTransmission(slaveAdr);
  Wire.write(regAddress);
  Wire.endTransmission(false); 
  
  // read out the data from the register
  Wire.requestFrom(slaveAdr, (uint8_t)2);

  // wait data are ready
  while(Wire.available() < 2){}

  msb = Wire.read();
  lsb = Wire.read();

  return (msb << 8) | lsb;
}

/*---------------------------------------------------------
Name:           readRegister

Beschreibung:   liest ein Register des I2C-LS
 
Eingnge:       I2C_LSRegister_t regAddress
                  Registeradresse von der die Daten gelesen
                  werden sollen

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readRegister(I2C_LSRegister_t regAddress)
{
  uint8_t lsb;
  uint8_t msb;
  
  Wire.beginTransmission(slaveAdr);
  Wire.write((uint8_t)regAddress);
  Wire.endTransmission(false); 
  
  // read out the data from the register
  Wire.requestFrom(slaveAdr, (uint8_t)2);

  // wait data are ready
  while(Wire.available() < 2){}

  msb = Wire.read();
  lsb = Wire.read();

  return (msb << 8) | lsb;
}

/*---------------------------------------------------------
Name:           writeConfigReg

Beschreibung:   schreibt das Config-Register des I2C-LS
 
Eingnge:       I2C_LSConfig conf
                  Config-Daten die geschrieben werden sollen

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::writeConfigReg(I2C_LSConfig conf)
{
  writeRegister(I2C_LS_REG_CONFIG, conf.config);
}

/*---------------------------------------------------------
Name:           writeLowLimitReg

Beschreibung:   schreibt Register LowLimit des I2C-LS
 
Eingnge:       uint16_t lowlim
                  LowLimit die geschrieben werden sollen

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::writeLowLimitReg(uint16_t lowlim)
{
  writeRegister(I2C_LS_REG_LOWLIMIT, lowlim);
}

/*---------------------------------------------------------
Name:           writeLowLimitLux

Beschreibung:   schreibt einen Lux Wert in das LowLimit des I2C-LS
 
Eingnge:       float lowlim value
                  HighLimit das geschrieben werden sollen

Ausgang:        uint16 lux 
                 als 16-bit Registerwert aus Exponent und Mantisse
---------------------------------------------------------*/
void I2C_Lichtsensor::writeLowLimitLux(float lowlim)
{
  uint16_t value;
  value = CovertU32ToExpMan(uint32_t(lowlim*100));
  writeRegister(I2C_LS_REG_LOWLIMIT, value);
}

/*---------------------------------------------------------
Name:           writeHighLimitReg

Beschreibung:   schreibt Register HighLimit des I2C-LS
 
Eingnge:       uint16_t highlim
                  HighLimit das geschrieben werden sollen

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::writeHighLimitReg(uint16_t highlim)
{
  writeRegister(I2C_LS_REG_HIGHLIMIT, highlim);
}

/*---------------------------------------------------------
Name:           writeHighLimitLux

Beschreibung:   schreibt einen Lux Wert in das HighLimit des I2C-LS
 
Eingnge:       float highlim value
                  HighLimit das geschrieben werden sollen

Ausgang:        uint16 lux 
                 als 16-bit Registerwert aus Exponent und Mantisse
---------------------------------------------------------*/
void I2C_Lichtsensor::writeHighLimitLux(float highlim)
{
  uint16_t value;
  value = CovertU32ToExpMan(uint32_t(highlim*100));
  writeRegister(I2C_LS_REG_HIGHLIMIT, value);
}

/*---------------------------------------------------------
Name:           writeConfigReg

Beschreibung:   schreibt das Config-Register des I2C-LS
 
Eingnge:       uint16_t conf
                  Config-Daten die geschrieben werden sollen

Ausgang:        void
---------------------------------------------------------*/
void I2C_Lichtsensor::writeConfigReg(uint16_t conf)
{
  writeRegister(I2C_LS_REG_CONFIG, conf);
}

/*---------------------------------------------------------
Name:           readLuxFixpoint

Beschreibung:   liest die Helligkeit des I2C-LS in Lux
                mit zwei festen Nachkommastellen
 
Eingnge:

Ausgang:        uint32_t data
                  gelesene Daten
---------------------------------------------------------*/
uint32_t I2C_Lichtsensor::readLuxFixpoint(void)
{
  /* Register Wert*/
  uint16_t reg_value = readResultReg();

  uint16_t mantisse;
  uint16_t exponent;

  /* Convert to LUX */
  mantisse = reg_value & 0x0fff;
  exponent = (reg_value & 0xf000) >> 12;

  return (uint32_t)((uint32_t)mantisse << exponent);// * 0.01;
}

/*---------------------------------------------------------
Name:           readLuxFloat

Beschreibung:   liest die Helligkeit des I2C-LS in Lux
                als FloatWert 
Eingnge:

Ausgang:        uint32_t data
                  gelesene Daten
---------------------------------------------------------*/
float I2C_Lichtsensor::readLuxFloat(void)
{
  /* Register Wert*/
  uint16_t reg_value = readResultReg();

  uint16_t mantisse;
  uint16_t exponent;

  /* Convert to LUX */
  mantisse = reg_value & 0x0fff;
  exponent = (reg_value & 0xf000) >> 12;

  return (float)((uint32_t)mantisse << exponent) * 0.01;
}

/*---------------------------------------------------------
Name:           readResultReg

Beschreibung:   liest das Register Result des I2C-LS
 
Eingnge:

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readResultReg()
{
  return readRegister(I2C_LS_REG_RESULT);
}

/*---------------------------------------------------------
Name:           readConfig

Beschreibung:   liest die Configuration des I2C-LS
 
Eingnge:

Ausgang:        I2C_LSConfig data
                  gelesene Config
---------------------------------------------------------*/
I2C_LSConfig I2C_Lichtsensor::readConfig()
{
  I2C_LSConfig temp_conf;
  temp_conf.config = readRegister(I2C_LS_REG_CONFIG);
  return temp_conf;
}

/*---------------------------------------------------------
Name:           readConfigReg

Beschreibung:   liest das Register Config des I2C-LS
 
Eingnge:

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readConfigReg()
{
  return readRegister(I2C_LS_REG_CONFIG);
}

/*---------------------------------------------------------
Name:           readLowLimitReg

Beschreibung:   liest das Register LowLimit des I2C-LS
 
Eingnge:

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readLowLimitReg()
{
  return readRegister(I2C_LS_REG_LOWLIMIT);
}

/*---------------------------------------------------------
Name:           readHighLimitReg

Beschreibung:   liest das Register HighLimit des I2C-LS
 
Eingnge:

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readHighLimitReg()
{
  return readRegister(I2C_LS_REG_HIGHLIMIT);
}

/*---------------------------------------------------------
Name:           readManufacturerId

Beschreibung:   liest die ManufacturerID des I2C-LS
 
Eingnge:

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readManufacturerId()
{
  return readRegister(I2C_LS_REG_MANUFACTURERID);
}

/*---------------------------------------------------------
Name:           readDeviceId

Beschreibung:   liest die DeviceID des I2C-LS
 
Eingnge:

Ausgang:        uint16_t data
                  gelesene Daten
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::readDeviceId()
{
  return readRegister(I2C_LS_REG_DEVICEID);
}

/*---------------------------------------------------------
Name:           CovertU32ToExpMan

Beschreibung:   wandelt Festkomma Luxwert in 16Bit Registerwert
                 aus Exponent und Mantisse
 
Eingnge:       uint32_t value
                  Festkomma Luxwert

Ausgang:        uint16_t data
                  Registerwert aus Exponent und Mantisse
---------------------------------------------------------*/
uint16_t I2C_Lichtsensor::CovertU32ToExpMan(uint32_t value)
{
  uint16_t ret;
  uint8_t exponent;
  uint16_t mantisse;
  for(exponent = 0; exponent<16 ; exponent++)
  {
    if( ((1 << (uint32_t)exponent) * 0x0fff) > value )
    {
      mantisse = (value / (1 << (uint32_t)exponent));
      break;
    }
  }
  if( exponent > 11 )
  {
    exponent = 11;
    mantisse = 0x0fff;
  }
  ret = ((uint16_t)(exponent & 0x0f) << 12) | (mantisse & 0x0fff);
  return ret;
}

/*---------------------------------------------------------
Name:           CovertExpManToFloat

Beschreibung:   wandelt 16Bit Registerwert aus Exponent und 
                 Mantisse in Lux als FliekommaWert
 
Eingnge:       uint16_t value
                  Registerwert aus Exponent und Mantisse
                  Festkomma Luxwert

Ausgang:        float data
                  Lux als FliekommaWert
---------------------------------------------------------*/
float I2C_Lichtsensor::CovertExpManToFloat(uint16_t value)
{
  /* Register Wert*/
  uint16_t reg_value = value;

  uint16_t mantisse;
  uint16_t exponent;

  /* Convert to LUX */
  mantisse = reg_value & 0x0fff;
  exponent = (reg_value & 0xf000) >> 12;

  return (float)((uint32_t)mantisse << exponent) * 0.01;
}

/*---------------------------------------------------------
Name:           CovertExpManToU32

Beschreibung:   wandelt 16Bit Registerwert aus Exponent und 
                 Mantisse in Festkomma Luxwert
 
Eingnge:       uint16_t value
                  Registerwert aus Exponent und Mantisse
                  

Ausgang:        uint32_t data
                  Festkomma Luxwert
---------------------------------------------------------*/
uint32_t I2C_Lichtsensor::CovertExpManToU32(uint16_t value)
{
  /* Register Wert*/
  uint16_t reg_value = value;

  uint16_t mantisse;
  uint16_t exponent;

  /* Convert to LUX */
  mantisse = reg_value & 0x0fff;
  exponent = (reg_value & 0xf000) >> 12;

  return ((uint32_t)mantisse << exponent);
}

/**********************************************************
Vorinstantiiertes Objekt
**********************************************************/
I2C_Lichtsensor I2C_LS = I2C_Lichtsensor();
