/******************************************************************************
Beispiel:      Interrupt

Beschreibung:  Dieses Beispiel zeigt die Verwendung des End of Conversion Interrupts
               
Hinweis:       Um End of Conversion Interrupts zu aktivieren muss das LowLimit
               Register mit 0xC000 beschreiben werden.
               Die I2C-SlaveAdresse des Lichtsensors kann bei der Initialisierung
               übergeben werden, folgende Werte sind möglich.
                I2C_LS_ADR_J1_GND 0x00 (default)
                I2C_LS_ADR_J1_VCC 0x00
                I2C_LS_ADR_J1_SCL 0x00
                I2C_LS_ADR_J1_SDA 0x00
               Wird keine Adresse angegeben wird die default verwendet.
               
               Für die Interrupts wird ein DigitalerPin vom Arduino benötigt, dabei
               können nur bestimmte Pins verwendet werden, mehr unter: 
               https://www.arduino.cc/reference/en/language/functions/external-interrupts/attachinterrupt/
******************************************************************************/
#include <stdio.h>
#include "I2C-LS.h"

/******************************************************************************
Definitionen
******************************************************************************/
const byte interruptPin = 2; //Bei Arduino Uno Pins D2 oder D3 nutzbar 

/******************************************************************************
globale Variablen
******************************************************************************/
/* Flag zur Anzeige eines Interrupts */
volatile uint8_t InterruptFlag = 0;

/******************************************************************************
Funktionen
******************************************************************************/
void setup()
{
  /* die serielle Ausgabe initialisieren */
  Serial.begin(115200);
  
  /* I2C-LS initialisieren mit I2C-Adresse*/  
  I2C_LS.begin( I2C_LS_ADR_J1_GND );
  
  /* den Pin für den Interrupt als Eingang und als externe Interrupt-Quelle 
  definieren */
  pinMode(interruptPin, INPUT);
  attachInterrupt(digitalPinToInterrupt(interruptPin), &ConversionReadyInterrupt, FALLING);
  
  /* Lese Manufacturer ID vom Lichtsensor */
  Serial.print("Manufacturer ID: "); 
  Serial.println(I2C_LS.readManufacturerId(),HEX);

  /* Lese Device ID vom Lichtsensor */
  Serial.print("Device ID: ");
  Serial.println(I2C_LS.readDeviceId(),HEX);
  
  /* manuell Reset des I2C-LS ausführen um möglich Configurationen zurückzusetzen */
  I2C_LS.reset();
    
  /* Initialisierung des EndOfConversion Modes */
  I2C_LS.writeRegister( I2C_LS_REG_LOWLIMIT, 0xC000 );
  
  /* Initialisierung im Continouous Mode */
  I2C_LS.writeConfigReg( I2C_LS_CONFIG_CONT_FULL_800MS );
  
  /* Interrupts einschalten */  
  interrupts();
}

void loop()
{
  /* wenn das Flag für den Interrupt gesetzt wird die Helligkeit aus dem
     I2C-LS auslesen und über die serielle Schnittstelle ausgegeben*/  
  if(InterruptFlag == 1)
  {
    /* Read I2C-LS */
    Serial.print("LUX:");
    Serial.println(I2C_LS.readLuxFloat(), 2);
    
    /* Clear the Interrupt Pin and Flags */ 
    I2C_LS.readConfigReg();
    InterruptFlag = 0;
  }
}

void ConversionReadyInterrupt(void)
{
  InterruptFlag = 1;
}